@@ -266,10 +266,26 @@ class SynapticPathway(CodeRunner, Group):
     @device_override('synaptic_pathway_update_abstract_code')
     def update_abstract_code(self, run_namespace=None, level=0):
         if self.synapses.event_driven is not None:
+            event_driven_eqs = self.synapses.event_driven
+            clock_driven_eqs = self.synapses.equations
             try:
-                event_driven_update = linear(self.synapses.event_driven,
+                event_driven_update = linear(event_driven_eqs,
                                              self.group.variables)
             except UnsupportedEquationsException:
+                # Check whether equations are independent
+                for var, expr in event_driven_eqs.diff_eq_expressions:
+                    for identifier in expr.identifiers:
+                        if identifier == var:
+                            continue
+                        if (identifier in event_driven_eqs.diff_eq_names or
+                                identifier in clock_driven_eqs):
+                            err = ("Cannot solve the differential equation for "
+                                   "'{}' as event-driven, it depends on "
+                                   "another variable '{}'. Use (clock-driven) "
+                                   "instead.".format(var,
+                                                                    identifier))
+                            raise UnsupportedEquationsException(err)
+                # All equations are independent, go ahead
                 event_driven_update = independent(self.synapses.event_driven,
                                                   self.group.variables)
             # TODO: Any way to do this more elegantly?
@@ -281,7 +297,8 @@ class SynapticPathway(CodeRunner, Group):
             self.abstract_code = ''
 
         self.abstract_code += self.code + '\n'
-        self.abstract_code += 'lastupdate = t\n'
+        if self.synapses.event_driven is not None:
+            self.abstract_code += 'lastupdate = t\n'
 
     @device_override('synaptic_pathway_before_run')
     def before_run(self, run_namespace):
@@ -442,6 +459,7 @@ def find_synapses(index, synaptic_neuron):
         for neuron in index:
             targets = np.flatnonzero(synaptic_neuron == neuron)
             synapses.extend(targets)
+        synapses = np.array(synapses, dtype=np.int32)
 
     return synapses
 
@@ -692,6 +710,11 @@ class Synapses(Group):
         Group.__init__(self, dt=dt, clock=clock, when='start', order=order,
                        name=name)
 
+        if dtype is None:
+            dtype = {}
+        if isinstance(dtype, collections.MutableMapping):
+            dtype['lastupdate'] = self._clock.variables['t'].dtype
+
         #: remember whether connect was called to raise an error if an
         #: assignment to a synaptic variable is attempted without a preceding
         #: connect.
@@ -727,13 +750,11 @@ class Synapses(Group):
                                               ('summed', 'shared',
                                                'constant over dt')])
 
-        for name in ['j', 'lastupdate', 'delay']:
+        for name in ['i', 'j', 'delay']:
             if name in model.names:
                 raise SyntaxError('"%s" is a reserved name that cannot be '
                                   'used as a variable name.' % name)
 
-        # Add the lastupdate variable, needed for event-driven updates
-        model = model + Equations('lastupdate : second')
         # Add the "multisynaptic index", if desired
         self.multisynaptic_index = multisynaptic_index
         if multisynaptic_index is not None:
@@ -745,8 +766,6 @@ class Synapses(Group):
         # constant over a time step or not
         model, constant_over_dt = extract_constant_subexpressions(model)
 
-        self._create_variables(model, user_dtype=dtype)
-
         # Separate the equations into event-driven equations,
         # continuously updated equations and summed variable updates
         event_driven = []
@@ -777,9 +796,12 @@ class Synapses(Group):
 
         if len(event_driven):
             self.event_driven = Equations(event_driven)
+            # Add the lastupdate variable, needed for event-driven updates
+            model += Equations('lastupdate : second')
         else:
             self.event_driven = None
 
+        self._create_variables(model, user_dtype=dtype)
         self.equations = Equations(continuous)
 
         if namespace is None:
@@ -1128,7 +1150,8 @@ class Synapses(Group):
             # Raise an error if a variable name is also used for a synaptic
             # variable (we ignore 'lastupdate' to allow connections from another
             # Synapses object)
-            if name in equations.names and name != 'lastupdate':
+            if (name in equations.names and name != 'lastupdate' and
+                    'summed' not in equations[name].flags):
                 error_msg = ('The pre-synaptic variable {name} has the same '
                              'name as a synaptic variable, rename the synaptic '
                              'variable ').format(name=name)
@@ -1154,7 +1177,8 @@ class Synapses(Group):
             # Raise an error if a variable name is also used for a synaptic
             # variable (we ignore 'lastupdate' to allow connections to another
             # Synapses object)
-            if name in equations.names and name != 'lastupdate':
+            if (name in equations.names and name != 'lastupdate' and
+                    'summed' not in equations[name].flags):
                 error_msg = ("The post-synaptic variable '{name}' has the same "
                              "name as a synaptic variable, rename the synaptic "
                              "variable ").format(name=name)
@@ -1172,7 +1196,7 @@ class Synapses(Group):
                 # Also add all the post variables without a suffix, but only if
                 # it does not have a post or pre suffix in the target group
                 # (which could happen when connecting to synapses)
-                if not name.endswith('_post') or name.endswith('pre'):
+                if not name.endswith('_post') or name.endswith('_pre'):
                     self.variables.add_reference(name, self.target, name,
                                                  index=index)
             except TypeError:
@@ -1311,7 +1335,7 @@ class Synapses(Group):
                     j = None
                     if isinstance(p, basestring):
                         p_dep = self._expression_index_dependence(p)
-                        if '_postsynaptic_idx' in p_dep:
+                        if '_postsynaptic_idx' in p_dep or '_iterator_idx' in p_dep:
                             j = ('_k for _k in range(N_post) '
                                  'if ({expr}) and '
                                  'rand()<{p}').format(expr=condition, p=p)
@@ -1548,7 +1572,7 @@ class Synapses(Group):
             if varname == 'i':
                 deps.add('_presynaptic_idx')
             elif varname == 'j':
-                deps.add('_postsynaptic_idx')
+                deps.add('_iterator_idx')
             elif varname in additional_indices:
                 deps.add(additional_indices[varname])
             else:
@@ -1594,12 +1618,17 @@ class Synapses(Group):
         additional_indices['_vectorisation_idx'] = '_iterator_idx'
 
         postsynaptic_condition = False
+        postsynaptic_variable_used = False
         if parsed['if_expression'] is not None:
             deps = self._expression_index_dependence(parsed['if_expression'],
                                                      additional_indices)
-            if '_postsynaptic_idx' in deps or '_iterator_idx' in deps:
+            if '_postsynaptic_idx' in deps:
+                postsynaptic_condition = True
+                postsynaptic_variable_used = True
+            elif '_iterator_idx' in deps:
                 postsynaptic_condition = True
         template_kwds['postsynaptic_condition'] = postsynaptic_condition
+        template_kwds['postsynaptic_variable_used'] = postsynaptic_variable_used
 
         abstract_code['setup_iterator'] += setupiter
         abstract_code['create_j'] += '_pre_idx = _raw_pre_idx \n'
